unit SendData;

interface

uses
  Windows, SysUtils, Classes, Messages,Dialogs;


type
  TCopyDataType = (cdtString = 0, cdtStream = 1, cdtRecord = 2);

  TReceivedStringEvent = procedure(Sender: TObject; Value: String) of object;
  TReceivedStreamEvent = procedure(Sender: TObject; Value: TMemoryStream) of object;
  TReceivedRecordEvent = procedure(Sender: TObject; Value: Pointer) of object;
  TWMCopyDataEvent = procedure(var Msg : TWMCopyData) of object;

  TSendData = class(TComponent)
  private
    { Private declarations }
    FClassNameSD:string;
    FClassTextSD:String;
    FOnReceivedString:TReceivedStringEvent;
    FOnReceivedStream:TReceivedStreamEvent;
    FOnReceivedRecord:TReceivedRecordEvent;
    FOnCopyData:TWMCopyDataEvent;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Action(Msg: TWMCopyData);
    function SendString(Value:String):integer;
    function SendStream(Value:TMemoryStream):integer;
    function SendRecord(Size:Cardinal;Value:Pointer):integer;
    function SendStringA(Wnd:HWND;Value:String):integer;
    function SendStreamA(Wnd:HWND;Value:TMemoryStream):integer;
    function SendRecordA(Wnd:HWND;Size:Cardinal;Value:Pointer):integer;
  published
    { Published declarations }
    property ClassNamed:string read FClassNameSD write FClassNameSD;
    property ClassText:string read FClassTextSD write FClassTextSD;
    property OnReceivedString:TReceivedStringEvent read FOnReceivedString write FOnReceivedString;
    property OnReceivedStream:TReceivedStreamEvent read FOnReceivedStream write FOnReceivedStream;
    property OnReceivedRecord:TReceivedRecordEvent read FOnReceivedRecord write FOnReceivedRecord;
    property OnCopyData:TWMCopyDataEvent read FOnCopyData write FOnCopyData;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Send Data', [TSendData]);
end;

constructor TSendData.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TSendData.Destroy;
begin
 inherited Destroy;
end;

procedure TSendData.Action(Msg : TWMCopyData);
var
  copyDataType : TCopyDataType;
  ms: TMemoryStream;
begin
  copyDataType := TCopyDataType(Msg.CopyDataStruct.dwData);
  case copyDataType of
    cdtString: if Assigned(FOnReceivedString) then
                 FOnReceivedString(self,PChar(Msg.CopyDataStruct.lpData));
    cdtStream: if Assigned(FOnReceivedStream) then
               begin
                 ms := TMemoryStream.Create;
                 try
                   ms.Write(Msg.copyDataStruct.lpData^, Msg.copyDataStruct.cbData);
                   ms.Position := 0;
                   FOnReceivedStream(self,ms);
                 finally
                   ms.Free;
                 end;
               end;
    cdtRecord: if Assigned(FOnReceivedRecord) then
                 FOnReceivedRecord(self,Msg.CopyDataStruct.lpData)
  end;
end;

function TSendData.SendString(Value:String):integer;
var
  Wnd:HWND;
  Cnsd:PAnsiChar;
  Ctsd:PAnsiChar;
begin
  if (Length(FClassNameSD) = 0) then Cnsd := nil else Cnsd := PChar(FClassNameSD);
  if (Length(FClassTextSD) = 0) then Ctsd := nil else Ctsd := PChar(FClassTextSD);
  Wnd := FindWindow(Cnsd,Ctsd);
  if (Wnd > 0) then Result := SendStringA(Wnd,Value) else Result := -1;
end;

function TSendData.SendStream(Value:TMemoryStream):integer;
var
  Wnd:HWND;
  Cnsd:PAnsiChar;
  Ctsd:PAnsiChar;
begin
  if (Length(FClassNameSD) = 0) then Cnsd := nil else Cnsd := PChar(FClassNameSD);
  if (Length(FClassTextSD) = 0) then Ctsd := nil else Ctsd := PChar(FClassTextSD);
  Wnd := FindWindow(Cnsd,Ctsd);
  if (Wnd > 0) then Result := SendStreamA(Wnd,Value) else Result := -1;
end;

function TSendData.SendRecord(Size:Cardinal;Value:Pointer):integer;
var
  Wnd:HWND;
  Cnsd:PAnsiChar;
  Ctsd:PAnsiChar;
begin
  if (Length(FClassNameSD) = 0) then Cnsd := nil else Cnsd := PChar(FClassNameSD);
  if (Length(FClassTextSD) = 0) then Ctsd := nil else Ctsd := PChar(FClassTextSD);
  Wnd := FindWindow(Cnsd,Ctsd);
  if (Wnd > 0) then Result := SendRecordA(Wnd,Size,Value) else Result := -1;
end;

function TSendData.SendStringA(Wnd:HWND;Value:String):integer;
var
  copyDataStruct : TCopyDataStruct;
begin
   copyDataStruct.dwData := Integer(cdtString);
   copyDataStruct.cbData := 1 + Length(Value) ;
   copyDataStruct.lpData := PChar(Value) ;
   Result := SendMessage(Wnd, WM_COPYDATA, 0, Integer(@copyDataStruct))
end;

function TSendData.SendStreamA(Wnd:HWND;Value:TMemoryStream):integer;
var
  copyDataStruct : TCopyDataStruct;
begin
  copyDataStruct.dwData := Integer(cdtStream);
  copyDataStruct.cbData := Value.Size;
  copyDataStruct.lpData := Value.Memory;
  Result := SendMessage(Wnd, WM_COPYDATA, 0, Integer(@copyDataStruct))
end;

function TSendData.SendRecordA(Wnd:HWND;Size:Cardinal;Value:Pointer):integer;
var
  copyDataStruct : TCopyDataStruct;
begin
  copyDataStruct.dwData := Integer(cdtRecord);
  copyDataStruct.cbData := Size;
  copyDataStruct.lpData := Value;
  Result := SendMessage(Wnd, WM_COPYDATA, 0, Integer(@copyDataStruct))
end;

end.
 